import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  Input
} from '@angular/core';
import { ucapAnimations, DialogService } from '@ucap-webmessenger/ui';
import { Observable, Subscription, BehaviorSubject, combineLatest } from 'rxjs';
import {
  DeptInfo,
  QueryProtocolService,
  DeptSearchType,
  UserInfoSS
} from '@ucap-webmessenger/protocol-query';
import { Store, select } from '@ngrx/store';
import { NGXLogger } from 'ngx-logger';

import * as AppStore from '@app/store';
import * as QueryStore from '@app/store/messenger/query';
import * as StatusStore from '@app/store/messenger/status';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { LoginInfo, KEY_LOGIN_INFO, MainMenu } from '@app/types';
import { take, tap } from 'rxjs/operators';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { Company } from '@ucap-webmessenger/api-external';
import { TranslateService } from '@ngx-translate/core';

import { OrganizationService } from '@ucap-webmessenger/ui-organization';

@Component({
  selector: 'app-layout-chat-left-sidenav-organization-tree',
  templateUrl: './organization-tree.component.html',
  styleUrls: ['./organization-tree.component.scss'],
  animations: ucapAnimations
})
export class OrganizationTreeComponent implements OnInit, OnDestroy {
  @Input()
  gotoDeptTree: number;

  companyList$: Observable<Company[]>;
  companyCode: string;

  departmentInfoList$: Observable<DeptInfo[]>;
  myDepartmentUserInfoListSubscription: Subscription;

  loginInfo: LoginInfo;
  loginRes: LoginResponse;
  loginResSubscription: Subscription;

  isShowSearch = false;
  searchUserInfos: UserInfoSS[] = [];

  organizationTreeActivatedSubject: BehaviorSubject<
    boolean
  > = new BehaviorSubject<boolean>(false);
  organizationTreeActivatedSubscription: Subscription;

  constructor(
    private store: Store<any>,
    private sessionStorageService: SessionStorageService,
    private queryProtocolService: QueryProtocolService,
    private organizationService: OrganizationService,
    private logger: NGXLogger
  ) {
    this.loginInfo = this.sessionStorageService.get<LoginInfo>(KEY_LOGIN_INFO);
  }

  ngOnInit() {
    this.companyCode = this.loginInfo.companyCode;
    this.companyList$ = this.store.pipe(
      select(AppStore.SettingSelector.CompanySelector.companyList)
    );

    this.departmentInfoList$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.departmentInfoList)
    );

    this.loginResSubscription = this.store
      .pipe(
        take(1),
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        tap(loginRes => {
          this.loginRes = loginRes;
        })
      )
      .subscribe();

    this.myDepartmentUserInfoListSubscription = this.store
      .pipe(
        select(
          AppStore.MessengerSelector.QuerySelector.myDepartmentUserInfoList
        )
      )
      .subscribe(list => {
        if (!list) {
          this.store
            .pipe(
              take(1),
              select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
              tap(loginRes => {
                if (!!loginRes) {
                  this.store.dispatch(
                    QueryStore.selectedDept({
                      seq: loginRes.departmentCode,
                      name: loginRes.userInfo.deptName,
                      nameEn: loginRes.userInfo.deptNameEn,
                      nameCn: loginRes.userInfo.deptNameCn
                    })
                  );
                }
                this.loginRes = loginRes;
              })
            )
            .subscribe();
          return;
        }
      });

    this.organizationTreeActivatedSubscription = combineLatest([
      this.store.pipe(
        select(AppStore.MessengerSelector.SettingsSelector.gnbMenuIndex)
      ),
      this.store.pipe(
        select(
          AppStore.MessengerSelector.SettingsSelector.organizationTreeActivated
        )
      )
    ]).subscribe(([menu, activate]) => {
      this.organizationTreeActivatedSubject.next(
        menu === MainMenu.Organization || activate
      );
    });
  }

  ngOnDestroy(): void {
    if (!!this.loginResSubscription) {
      this.loginResSubscription.unsubscribe();
    }
    if (!!this.myDepartmentUserInfoListSubscription) {
      this.myDepartmentUserInfoListSubscription.unsubscribe();
    }
    if (!!this.organizationTreeActivatedSubscription) {
      this.organizationTreeActivatedSubscription.unsubscribe();
    }
  }

  /** 유저검색 */
  onKeyDownEnterOrganizationTenantSearch(params: {
    companyCode: string;
    searchWord: string;
  }) {
    if (params.searchWord.trim().length > 1) {
      this.store.dispatch(
        QueryStore.searchDeptUser({
          companyCode: params.companyCode,
          search: params.searchWord
        })
      );
    }
  }
  /** 검색 취소 */
  onClickCancel() {
    this.store.dispatch(QueryStore.clearSearchDeptUser({}));
  }

  /** 조직도 부서 선택 */
  onSelectedOrganization(deptInfo: DeptInfo) {
    this.onClickCancel();

    this.store.dispatch(
      QueryStore.selectedDept({
        seq: deptInfo.seq,
        name: deptInfo.name,
        nameEn: deptInfo.nameEn,
        nameCn: deptInfo.nameCn
      })
    );
  }
}
