import { Component, OnInit, Inject, OnDestroy } from '@angular/core';
import {
  FormGroup,
  FormBuilder,
  Validators,
  ValidatorFn,
  AbstractControl
} from '@angular/forms';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { GroupDetailData } from '@ucap-webmessenger/protocol-sync';
import { StringUtil } from '@ucap-webmessenger/core';
import { TranslateService, TranslateParser } from '@ngx-translate/core';
import { take, tap } from 'rxjs/operators';
import { Store, select } from '@ngrx/store';
import { Subscription } from 'rxjs';
import * as AppStore from '@app/store';

export interface EditGroupDialogData {
  title: string;
  group: GroupDetailData;
}

export interface EditGroupDialogResult {
  choice: boolean;
  groupName: string;
  group: GroupDetailData;
}

@Component({
  selector: 'app-layout-messenger-edit-group',
  templateUrl: './edit-group.dialog.component.html',
  styleUrls: ['./edit-group.dialog.component.scss']
})
export class EditGroupDialogComponent implements OnInit, OnDestroy {
  groupName: string;
  inputForm: FormGroup;

  bannedWords: string[] = [];
  groupListSubscription: Subscription;
  groupList: GroupDetailData[];

  constructor(
    private store: Store<any>,
    public dialogRef: MatDialogRef<EditGroupDialogData, EditGroupDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: EditGroupDialogData,
    private translateService: TranslateService,
    private translateParser: TranslateParser,
    private formBuilder: FormBuilder
  ) {
    const banKeys: string[] = [
      'group.nameFavorite',
      'group.nameMyDept',
      'group.nameDefault'
    ];

    const langs = ['ko', 'en'];

    langs.forEach(lang => {
      this.translateService
        .getTranslation(lang)
        .pipe(take(1))
        .subscribe(
          translation => {
            banKeys.forEach(banKey => {
              this.bannedWords.push(
                this.translateParser.getValue(translation, banKey)
              );
            });
          },
          error => {},
          () => {}
        );
    });
  }

  ngOnInit(): void {
    this.inputForm = this.formBuilder.group({
      groupName: [
        this.data.group.name,
        [
          Validators.required,
          StringUtil.checkSpecialCharacter(),
          this.checkBanWords(),
          this.checkSameName()
        ]
      ]
    });

    this.groupListSubscription = this.store
      .pipe(
        select(AppStore.MessengerSelector.SyncSelector.selectAllGroup2),
        tap(groupList => {
          this.groupList = groupList;
        })
      )
      .subscribe();
  }

  ngOnDestroy(): void {
    if (!!this.groupListSubscription) {
      this.groupListSubscription.unsubscribe();
    }
  }

  onClickChoice(choice: boolean): void {
    this.dialogRef.close({
      choice,
      groupName: this.inputForm.get('groupName').value,
      group: this.data.group
    });
  }

  checkBanWords(): ValidatorFn {
    return (control: AbstractControl): { [key: string]: any } | null => {
      if (!control || !control.value) {
        return null;
      }
      const ban =
        -1 < this.bannedWords.indexOf((control.value as string).trim());
      return ban ? { groupNameBanned: { value: control.value } } : null;
    };
  }

  checkSameName(): ValidatorFn {
    return (control: AbstractControl): { [key: string]: any } | null => {
      if (
        !control ||
        !control.value ||
        !this.groupList ||
        0 === this.groupList.length
      ) {
        return null;
      }
      const v = (control.value as string).trim();
      const ban = -1 < this.groupList.findIndex(g => g.name === v);
      return ban ? { groupNameSamed: { value: control.value } } : null;
    };
  }
}
